@@ -1,4 +1,4 @@
-## ---- include = FALSE---------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = TRUE,
   comment = "#>", warning = FALSE, 
@@ -15,6 +15,10 @@ library(kableExtra)
 library(WASP)
 library(ggplot2)
 
+if(!require(SPEI)) devtools::install_github('sbegueria/SPEI@v1.7.1') # use 1.7.1
+require(SPEI)
+library(readr)
+library(dplyr)
 library(FNN)
 library(synthesis)
 library(waveslim)
@@ -33,7 +37,7 @@ for (wf in c("haar", "d4", "d8", "d16")) {
   # wavelet family, extension mode and package
   # wf <- "haar" # wavelet family D8 or db4
   boundary <- "periodic"
-  if (wf != "haar") v <- as.integer(readr::parse_number(wf) / 2) else v <- 1
+  if (wf != "haar") v <- as.integer(parse_number(wf) / 2) else v <- 1
 
   # Maximum decomposition level J
   n <- length(x)
@@ -112,7 +116,7 @@ for (k in seq_along(wf.opts)) {
   # wavelet family, extension mode and package
   wf <- wf.opts[k] # wavelet family D8 or db4
   boundary <- "periodic"
-  if (wf != "haar") v <- as.integer(readr::parse_number(wf) / 2) else v <- 1
+  if (wf != "haar") v <- as.integer(parse_number(wf) / 2) else v <- 1
 
   # Maximum decomposition level J
   n <- length(x)
@@ -214,8 +218,9 @@ if (TRUE) {
   data("obs.mon")
   data("rain.mon")
 
-  if (1) { # SPI12 as response
-    SPI.12 <- SPEI::spi(rain.mon[, 5], scale = 12)$fitted
+  if (TRUE) { # SPI12 as response
+	#SPI.12 <- SPEI::spi(rain.mon[, 5], scale = 12)$fitted
+	SPI.12 <- SPI.calc(window(rain.mon[, 5], start=c(1949,1), end=c(2009,12)),sc=12)
     x <- window(SPI.12, start = c(1950, 1), end = c(2009, 12))
     dp <- window(obs.mon, start = c(1950, 1), end = c(2009, 12))
   } else { # rainfall as response
@@ -246,7 +251,7 @@ for (mode in mode.opts) {
   wf <- "haar"
   pad <- "zero"
   boundary <- "periodic"
-  if (wf != "haar") v <- as.integer(readr::parse_number(wf) / 2) else v <- 1
+  if (wf != "haar") v <- as.integer(parse_number(wf) / 2) else v <- 1
 
   # Maximum decomposition level J
   n <- sample
@@ -326,14 +331,15 @@ if (TRUE) {
   data.list <- lapply(ts.list, function(ts) list(x = ts$z, dp = cbind(ts$x, ts$y)))
 
   lab.names <- c("x", "y")
-  ylim <- c(-55, 55)
+  xlim<- c(0,n);ylim <- c(-55, 55)
 } else {
 
   ### Real-world example
   data("obs.mon")
   data("rain.mon")
 
-  SPI.12 <- SPEI::spi(rain.mon[, 5], scale = 12)$fitted
+  #SPI.12 <- SPEI::spi(rain.mon[, 5], scale = 12)$fitted
+  SPI.12 <- SPI.calc(window(rain.mon[, 5], start=c(1949,1), end=c(2009,12)),sc=12)
   x <- window(SPI.12, start = c(1950, 1), end = c(2009, 12))
   dp <- window(obs.mon, start = c(1950, 1), end = c(2009, 12))
 
@@ -341,7 +347,7 @@ if (TRUE) {
   sample <- length(x)
 
   lab.names <- colnames(obs.mon)
-  ylim <- NULL
+  xlim<- NULL;ylim <- NULL
 }
 
 #-------------------------------------------------------------------
@@ -370,14 +376,14 @@ for (mode in mode.opts) {
   wf <- "d16"
   pad <- "zero"
   boundary <- "periodic"
-  if (wf != "haar") v <- as.integer(readr::parse_number(wf) / 2) else v <- 1
+  if (wf != "haar") v <- as.integer(parse_number(wf) / 2) else v <- 1
 
   # Maximum decomposition level J
   n <- sample
   J <- ceiling(log(n / (2 * v - 1)) / log(2)) - 1 # (Kaiser, 1994)
   # J <- floor(log(n/(2*v-1))/log(2))
 
-  # variance transfrom - calibration
+  # variance transform - calibration
   if (mode == "MRA") {
     dwt.list <- lapply(data.list, function(x) dwt.vt(x, wf, J, method, pad, boundary, cov.opt, flag))
   } else if (mode == "MODWT") {
@@ -402,7 +408,7 @@ for (mode in mode.opts) {
     for (i in 1:ncol(dwt$dp)) {
       ts.plot(cbind(dwt$dp[, i], dwt$dp.n[, i]),
         xlab = NA, ylab = paste0(lab.names[i]),
-        xlim = c(0, n), ylim = ylim,
+        xlim = xlim, ylim = ylim,
         col = c("black", "blue"), lwd = c(1, 2)
       )
     }
@@ -428,7 +434,8 @@ station.id <- 5
 lab.names <- colnames(obs.mon)[c(1, 3, 4, 5, 7)]
 
 if (TRUE) { # SPI12 as response
-  SPI.12 <- SPEI::spi(rain.mon, scale = 12)$fitted
+  #SPI.12 <- SPEI::spi(rain.mon, scale = 12)$fitted
+  SPI.12 <- SPI.calc(window(rain.mon, start=c(1949,1), end=c(2009,12)),sc=12)
   x <- window(SPI.12, start = c(1950, 1), end = c(2009, 12))
   dp <- window(obs.mon[, lab.names], start = c(1950, 1), end = c(2009, 12))
 } else { # rainfall as response
@@ -469,7 +476,7 @@ for (mode in mode.opts) {
   )
   pad <- "zero"
   boundary <- "periodic"
-  if (wf != "haar") v <- as.integer(readr::parse_number(wf) / 2) else v <- 1
+  if (wf != "haar") v <- as.integer(parse_number(wf) / 2) else v <- 1
 
   # Maximum decomposition level J
   n <- nrow(x)
@@ -490,7 +497,7 @@ for (mode in mode.opts) {
       MSE <- rbind(MSE, c(m1, m2))
     }
 
-    RMSE <- cbind(RMSE, MSE)
+    RMSE <- rbind(RMSE, data.frame(mode, MSE))
 
     par(
       mfrow = c(length(cpy), 1), mar = c(0, 4, 2, 1),
@@ -517,18 +524,22 @@ par(op)
 # plot and save
 cowplot::plot_grid(plotlist = p.list, nrow = 1, labels = c("(a)", "(b)", "(c)"))
 
-
-
 #-------------------------------------------------------------------
 # RMSE when more predictors are included
-tab1 <- round(RMSE, 3)
-tab1 <- cbind(1:nrow(tab1), tab1)
-colnames(tab1) <- c("No. of Predictors", rep(c("Original", "Transformed"), length(mode.opts)))
-
-kable(tab1, caption = "Comparison of prediction accuracy using Std and SVT", booktabs = T) %>%
+#tab1 <- round(RMSE, 3)
+#tab1 <- cbind(1:nrow(tab1), tab1)
+#colnames(tab1) <- c("No. of Predictors", rep(c("Original", "Transformed"), length(mode.opts)))
+# kable(tab1, caption = "Comparison of prediction accuracy using Std and SVT", booktabs = T) %>%
+#   kable_styling(latex_options = c("HOLD_position"), position = "center", full_width = FALSE)  %>%
+#   #  add_header_above(c(" " = 1, "DWT-MRA" = 2, "MODWT" = 2, "AT" = 2))
+#   add_header_above(c(" " = 1, "DWT-MRA" = 2, "MODWT/AT" = 2))
+tab1 <- RMSE %>% group_by(mode) %>% mutate(id = row_number())
+colnames(tab1) <- c("Method","No. of Predictors","Original","Transformed")
+kable(tab1[,c(1,4,2,3)], caption = "Comparison of prediction accuracy using Std and SVT", booktabs = T, 
+      digits = 3) %>%
   kable_styling(latex_options = c("HOLD_position"), position = "center", full_width = FALSE)  %>%
-  #  add_header_above(c(" " = 1, "DWT-MRA" = 2, "MODWT" = 2, "AT" = 2))
-  add_header_above(c(" " = 1, "DWT-MRA" = 2, "MODWT/AT" = 2))
+  collapse_rows(columns = 1)
+
 
 ## ----comp, eval=FALSE, include=FALSE------------------------------------------
 #  #-------------------------------------------------------------------
@@ -579,7 +590,7 @@ kable(tab1, caption = "Comparison of prediction accuracy using Std and SVT", boo
 #    #if(wf=="haar"&&mode=="MODWT") J = J-1 #since modwt no need a dyadic number size
 #    print(paste0("Calibration: Decomposition Levels J= ",J))
 #  
-#    #variance transfrom
+#    #variance transform
 #    if(mode=="MRA"){
 #      dwt.list<- lapply(data.list, function(x) dwt.vt(x, wf, J, method, pad, boundary, cov.opt))
 #    } else if(mode=="MODWT") {
